import os
import re
import subprocess
import logging

from .conf import settings
from .exceptions import FileTypeNotSupported

"""
Uses command line pdfinfo utility (from poppler pakage) for various
small operations (e.g. get pdf page count).
"""

logger = logging.getLogger(__name__)


def get_tiff_pagecount(filepath):
    cmd = [
        settings.BINARY_IDENTIFY,
        "-format",
        "%n\n",
        filepath
    ]
    compl = subprocess.run(
        cmd,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE
    )

    if compl.returncode:

        logger.error(
            "get_tiff_pagecount: cmd=%s args=%s stdout=%s stderr=%s code=%s",
            cmd,
            compl.args,
            compl.stdout,
            compl.stderr,
            compl.returncode,
            stack_info=True
        )

        raise Exception("Error occured while getting document page count.")

    lines = _split(stdout=compl.stdout)
    # look up for the line containing "Pages: 11"
    for line in lines:
        x = re.match(r"(\d+)", line.strip())
        if x:
            return int(x.group(1))

    return 0


def get_pagecount(filepath):
    """
    Returns the number of pages in a PDF document as integer.

    filepath - is filesystem path to a PDF document
    """
    if not os.path.isfile(filepath):
        raise ValueError("Filepath %s is not a file" % filepath)

    if os.path.isdir(filepath):
        raise ValueError("Filepath %s is a directory!" % filepath)

    base, ext = os.path.splitext(filepath)

    # pure images (png, jpeg) have only one page :)
    if ext and ext.lower() in ('.jpeg', '.png', '.jpg'):
        # whatever png/jpg image is there - it is
        # considered by default one page document.
        return 1

    if ext and ext.lower() in ('.tiff', ):
        return get_tiff_pagecount(filepath)

    if ext and ext.lower() not in ('.pdf', '.tiff'):
        raise FileTypeNotSupported(
            "Only jpeg, png, pdf and tiff are handlerd by this"
            " method"
        )
    # pdfinfo "${PDFFILE}" | grep Pages
    cmd = [
        settings.BINARY_PDFINFO,
        filepath
    ]
    compl = subprocess.run(
        cmd,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE
    )

    if compl.returncode:

        logger.error(
            "get_pagecount: cmd=%s args=%s stdout=%s stderr=%s code=%s",
            cmd,
            compl.args,
            compl.stdout,
            compl.stderr,
            compl.returncode,
            stack_info=True
        )

        raise Exception("Error occured while getting document page count.")

    lines = _split(stdout=compl.stdout)
    # look up for the line containing "Pages: 11"
    for line in lines:
        x = re.match(r"Pages:\W+(\d+)$", line.strip())
        if x:
            return int(x.group(1))

    return 0


def _split(stdout):
    """
    stdout is result.stdout where result
    is whatever is returned by subprocess.run
    """
    decoded_text = stdout.decode(
        'utf-8',
        # in case there are decoding issues, just replace
        # problematic characters. We don't need text verbatim.
        'replace'
    )
    lines = decoded_text.split('\n')

    return lines



import os
import logging

from mglib.runcmd import run
from .conf import settings

logger = logging.getLogger(__name__)


def convert_tiff2pdf(doc_url):

    logger.debug(f"convert_tiff2pdf for {doc_url}")
    # basename is filename + ext (no path)

    basename = os.path.basename(doc_url)
    base_root, base_ext = os.path.splitext(basename)
    root, ext = os.path.splitext(doc_url)
    new_doc_url = f"{root}.pdf"

    logger.debug(
        f"tiff2pdf source={doc_url} dest={new_doc_url}"
    )

    cmd = (
        settings.BINARY_CONVERT,
        doc_url,
        new_doc_url,
    )

    run(cmd)

    # returns new filename
    return f"{base_root}.pdf"



class Step:

    # width of a document when displayed as 100%.
    WIDTH_100p = 1240
    PERCENT = 100
    LIST = [125, 100, 75, 50, 10]

    # aspect ration for A4 paper is h = w * 1.41
    # for 100
    # 100 => w = 1240, h = 1748
    # 50  => w = 620, h = 874

    def __init__(self, current=1):
        self.current = current

    @property
    def width(self):
        p = self.percent / 100
        return int(p * Step.WIDTH_100p)

    @property
    def is_thumbnail(self):
        return self.percent < 50

    @property
    def is_for_hocr(self):
        return not self.is_thumbnail

    @property
    def percent(self):
        return Step.LIST[self.current]

    def __str__(self):
        return f"Step(percent={self.percent}, width={self.width})"

    def __repr__(self):
        return self.__str__()


class Steps:
    def __init__(self):
        self.steps = [Step(0), Step(1), Step(2), Step(3), Step(4)]

    def __iter__(self):
        return iter(self.steps)
